package com.iona.fuse.demo.logisticx.order.processor;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.annotation.Resource;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import com.iona.fuse.demo.logisticx.model.LineItem;
import com.iona.fuse.demo.logisticx.model.Order;
import com.iona.fuse.demo.logisticx.model.OrderStatus;
import com.iona.fuse.demo.logisticx.order.processor.support.OrderProcessorSupport;

import org.apache.servicemix.jbi.listener.MessageExchangeListener;

import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.iona.fuse.demo.logisticx.order.processor.support.SpringJmsConsumer;
import com.iona.fuse.demo.logisticx.order.processor.support.SpringJmsProducer;


/**
 * The order processor performs asynchronous processing of orders. It receives a JBI 
 * message exchange containing the order identifier to be processed. 
 * 
 * @author bsnyder
 *
 */

public class OrderProcessor implements MessageExchangeListener, ApplicationContextAware {

    /** The JBI <a href="http://servicemix.apache.org/dist/servicemix-3.2.1/site/parent/core/servicemix-jbi/apidocs/javax/jbi/component/ComponentContext.html">
     * component context</a> to provide access to items such as the {@link DeliveryChannel}, {@link ServiceEndpoint}s, 
     * the endpoint registry and other important JBI 
     */ 
	@Resource private ComponentContext context;

    private static final Log log = LogFactory.getLog(OrderProcessor.class);

    /** The namespace to use for calling any other services **/
    private static final String LOGISTICX_NAMESPACE = "http://logisticx.demo.fuse.iona.com/";

    /** The service name of the cbr service **/
    private static final String CBR_SERVICE_NAME = "cbrService";
    
    /** The name of the order processor endpoint on the processor service **/
    private static final String CBR_ENDPOINT_NAME = "cbrEndpoint";        

    /** 
     * The JBI delivery channel is bi-directional and is used to communicate
     * with the normalized message rouer. 
     */ 
    @Resource
    private DeliveryChannel channel;
    
    /** The support object handles the heavy lifting. **/
    private OrderProcessorSupport support = new OrderProcessorSupport();

    private ApplicationContext ctx;
    private SpringJmsProducer producer;
    
    private SpringJmsConsumer consumer;
    
    public ComponentContext getContext() {
        return context;
    }

    public void setApplicationContext(ApplicationContext applicationContext)
            throws BeansException {
        ctx = applicationContext;
    }

    public void setConsumer(SpringJmsConsumer jmsConsumer) {
        consumer = jmsConsumer;
        consumer.setOrderProcessorSupport(support);
    }

    public void setProducer(SpringJmsProducer jmsProducer) {
        producer = jmsProducer;
        support.setProducer(producer);    
    }
    
    /**
     * Receive the message exchange.                                                                
     */
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {        
        if (exchange.getStatus() == ExchangeStatus.ACTIVE) {            
            log.info("Received exchange: " + exchange);
            
            OrderStatus status = null;
                        
            // 1. send to payment service 
            
            // 2. send to Validation Service 
            
            // 3. send to Warehouse Service - calls instock to see if the item is there.
            status = ProcessWarehouseService(exchange);
            
            // 4. Send to Shipping Service
            
            // 5. Send to Delivery Service                        
                                                
        } else {
            log.info("Received exchange but status not ACTIVE: " + exchange);                   
        }        
            
    }
    
    /*
     * Sends request to a Camel content based router Service that will 
     * decide which external warehouse service to send it to.
     * 
     */
    private OrderStatus ProcessWarehouseService(MessageExchange exchange) throws MessagingException {
    	OrderStatus status = null;
    	long orderId;
    	
    	InOnly inOnly =  (InOnly)exchange;
        NormalizedMessage in = inOnly.getInMessage();                       
        SourceTransformer sourceTransformer = new SourceTransformer();        

        QName cbrServiceName = new QName(LOGISTICX_NAMESPACE, CBR_SERVICE_NAME);
        ServiceEndpoint se = getContext().getEndpoint(cbrServiceName, CBR_ENDPOINT_NAME);

        InOut cbrInOut = channel.createExchangeFactory().createInOutExchange();                        
        cbrInOut.setEndpoint(se);
        
        orderId = support.parseOrderId(in);
        NormalizedMessage msg = cbrInOut.createMessage();            
        try {
           msg.setContent(getOrderContent(orderId));
        } catch (Exception ex) {
                ex.printStackTrace();
        }

        QName opName =  new QName("http://logisticx.demo.fuse.iona.com/warehouseService/", "inStock");            
        cbrInOut.setOperation(opName);
        cbrInOut.setMessage(msg, "in");
        cbrInOut.setService(cbrServiceName);       
        channel.sendSync(cbrInOut);                        
        try {
            // gets the response from the warehouse service
            // parse the status from the message and updates the db
            // may also send a message to a queue            
            status = support.parseOrderStatus(cbrInOut.getOutMessage()); 
            if (status.getStatusCode().equals(OrderStatus.StatusCode.ORDER_INSTOCK)) {
                status = support.update(orderId, status.getStatusCode(), "Your order " + orderId + " is in Stock");
            } else if (status.getStatusCode().equals(OrderStatus.StatusCode.ORDER_AWAITINGSTOCK)) {
                status = support.update(orderId, status.getStatusCode(), "Your order " + orderId + " is awaiting Stock");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }                
     
        return status;

    }
    
    /*
     * Sets up the content of the message - for the instock method
     *
     */
    private StringSource getOrderContent(long orderId) {
    	    	
    	Order order = support.getOrder(orderId);    	    	
    	LineItem lineItem = order.getLineItem();
        
        StringBuffer buf = new StringBuffer();
        buf.append("<ns1:inStock xmlns:ns1='http://logisticx.demo.fuse.iona.com/warehouseService/'>");
        buf.append("<arg0 xmlns:ns2='http://logisticx.demo.fuse.iona.com/warehouseService/'>");
        buf.append("<customerId>");
        buf.append(order.getCustomerId());
        buf.append("</customerId><customerPoNumber>");
        buf.append(order.getCustomerPoNumber());
        buf.append("</customerPoNumber><lineItem><description>");
        buf.append(lineItem.getDescription());
        buf.append("</description><itemId>");
        buf.append(lineItem.getItemId());
        buf.append("</itemId><lineItemId>");
        buf.append(lineItem.getLineItemId());
        buf.append("</lineItemId><price>");
        buf.append(lineItem.getPrice());
        buf.append("</price><quantity>");
        buf.append(lineItem.getQuantity());
        buf.append("</quantity><totalPrice>");
        buf.append(lineItem.getTotalPrice());
        buf.append("</totalPrice></lineItem>");        
        buf.append("<status><statusCode>ORDER_PLACED</statusCode></status></arg0></ns1:inStock>");
      
        return new StringSource(buf.toString());    	
    }

}
