package com.iona.fuse.demo.logisticx.order.processor.support;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import java.io.StringReader;
import java.util.Iterator;
import javax.xml.namespace.NamespaceContext;

import javax.annotation.Resource;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.NormalizedMessage;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.xpath.*;

import com.iona.fuse.demo.logisticx.model.Order;
import com.iona.fuse.demo.logisticx.model.OrderStatus;
import com.iona.fuse.demo.logisticx.model.OrderStatus.StatusCode;

import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.xml.sax.InputSource;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.support.FileSystemXmlApplicationContext;

import com.iona.fuse.demo.logisticx.order.processor.support.SpringJmsConsumer;
import com.iona.fuse.demo.logisticx.order.processor.support.SpringJmsProducer;

/** 
 * A helper class that is the worker bee for the order processor. 
 * 
 * @author bsnyder
 *
 */
public class OrderProcessorSupport {

    private static final String XPATH_ORDER_ID = "/order-id/text()";
    private static final String XPATH_ORDER_STATUS = "/status/text()";
    private static final String CONFIG = "xbean.xml";       
    private SpringJmsProducer producer;  
    
    @Resource private ComponentContext context;

    private ApplicationContext ctx;

    /** A JPA query to retrieve the order status based on the order id. **/    
    private static final String ORDERSTATUS_QUERY = 
        "select stat from com.iona.fuse.demo.logisticx.model.OrderStatus stat where stat.order.orderId = ?1";

    /** A JPA query to retrieve the order status based on the order id. **/    
    private static final String ORDER_QUERY = 
        "select ord from com.iona.fuse.demo.logisticx.model.Order ord where ord.orderId = ?1";

    public OrderProcessorSupport() {
    }

    public void setProducer(SpringJmsProducer jmsProducer) {
        producer = jmsProducer;
    }
            
    public OrderStatus getOrderStatus(long orderId) {
        return doJpaRetrieveOrderStatus(orderId);
    }

    public boolean updateStatus(long orderId, StatusCode statusCode, String comments) {
        OrderStatus status = doJpaRetrieveUpdateOrderStatus(orderId, statusCode, comments);
        if (status != null) {
        	return true;
        }
        return false;
    }
    
    /**
     * Query for an order based on the given <tt>orderId</tt> and return 
     * an OrderStatus object. To be replaced with <a href="http://activemq.apache.org/camel/jpa.html">
     * Camel JPA</a>.
     * 
     * @param orderId
     * @return The status of the order 
     * The <a href="http://java.sun.com/javaee/5/docs/api/javax/persistence/EntityManagerFactory.html">
     * entity manager factory</a> for the JPA persistence. See {@link #doJpaPersist(Order)}. 
     *     
     */
    private OrderStatus doJpaRetrieveOrderStatus(Long orderId) {
        OrderStatus status = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("order-processor");
        EntityManager em = emFactory.createEntityManager();

        try {
            em.getTransaction().begin();                       
            Query orderQuery = em.createQuery(ORDERSTATUS_QUERY);
            orderQuery.setParameter(1, orderId);
            status = (OrderStatus) orderQuery.getSingleResult();
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return status;
    }

    /**
     * Retrieve the Customers order from the database
     * based on the given <tt>orderId</tt>
     * 
     * @param orderId
     * @return update Order
     *
     * The <a href="http://java.sun.com/javaee/5/docs/api/javax/persistence/EntityManager.html">
       * entity manager</a> for the JPA persistence. See {@link #doJpaPersistOrder(Order)}.     
     */
    public Order getOrder(Long orderId) {
    	Order order = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("order-processor");
    
        EntityManager em = emFactory.createEntityManager();

        try {
            em.getTransaction().begin();                       
            Query orderQuery = em.createQuery(ORDER_QUERY);
            orderQuery.setParameter(1, orderId);
            order = (Order) orderQuery.getSingleResult();
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return order;
    }

    
    /**
     * Query for an orderStatus based on the given <tt>orderId</tt> and update 
     * the OrderStatus object to be ORDER_FULFILLED. To be replaced with 
     * <a href="http://activemq.apache.org/camel/jpa.html">
     * Camel JPA</a>.
     * 
     * @param orderId
     * @return update success/fail 
     */
    private OrderStatus doJpaRetrieveUpdateOrderStatus(Long orderId, StatusCode statusCode,
    		String comments) {
        OrderStatus status = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("order-processor");
        EntityManager em = emFactory.createEntityManager();
        
        try {            
            em.getTransaction().begin();
            Query orderQuery = em.createQuery(ORDERSTATUS_QUERY);
            orderQuery.setParameter(1, orderId);
            status = (OrderStatus) orderQuery.getSingleResult();
            status.setStatusCode(statusCode);
            status.setComments(comments);
            em.merge(status);
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return status;
    }

    
    /**
     * Parse the <tt>orderId</tt> from an XML message that looks like this: 
     * <p>
     *     <order-id>1234</order-id>
     * 
     * @param message
     * @return
     */
    public long parseOrderId(NormalizedMessage message) {
        SourceTransformer transformer = new SourceTransformer();
        String orderIdStr = null;
        
        try {
            // Transform the message content into a InputSource object 
            InputSource  orderIdXml = new InputSource(new StringReader(transformer.contentToString(message)));
            XPathFactory xpathFact = XPathFactory.newInstance();
            // Parse the order id from the XML 
            XPath xpath = xpathFact.newXPath();                  
            orderIdStr = xpath.evaluate(XPATH_ORDER_ID, orderIdXml);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new Long(orderIdStr).longValue();
    }
    
    public OrderStatus parseOrderStatus(NormalizedMessage message) {
        OrderStatus status = new OrderStatus();
    	SourceTransformer transformer = new SourceTransformer();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        
        try {      
            InputSource  src = new InputSource(new StringReader(transformer.contentToString(message)));

            NamespaceContext ctx = new NamespaceContext() {
                      public String getNamespaceURI(String prefix) {
                          String uri;
                          if (prefix.equals("soap")) {
                        	  uri = "http://schemas.xmlsoap.org/soap/envelope/"; 
                          } else if (prefix.equals("ns2")) {                             
                             uri = "http://logisticx.demo.fuse.iona.com/warehouseService/";
                          } else
                              uri = null;
                          return uri;
                      }
                      
                      // Dummy implementation - not used!
                      public Iterator getPrefixes(String val) {
                          return null;
                      }
                      
                      // Dummy implemenation - not used!
                      public String getPrefix(String uri) {
                          return null;
                      }
                  };
           
                  String xpathStr = "/soap:Envelope/soap:Body/ns2:inStockResponse/return/statusCode";
                  XPathFactory xpathFact = XPathFactory.newInstance();
                  XPath xpath = xpathFact.newXPath();
                  xpath.setNamespaceContext(ctx);
                  
                  String result = xpath.evaluate(xpathStr, src);
                  if (result.equals("ORDER_INSTOCK")) {                	   
                	  status.setStatusCode(StatusCode.ORDER_INSTOCK);
                  } else if (result.equals("ORDER_AWAITINGSTOCK")) {
                	  status.setStatusCode(StatusCode.ORDER_AWAITINGSTOCK);
                  }
                  System.out.println("result " +  result);
                  
         } catch (Exception ex) {
                ex.printStackTrace();
         }      
        
        return status;
    }
    
    public OrderStatus update(long orderId, StatusCode code, String comments) {
        final long id = orderId;
        final StatusCode codeStatus = code;
        
        OrderStatus status = doJpaRetrieveUpdateOrderStatus(orderId, code, comments);
        // send message to client
        updateClientStatus(status);
        return status;
    }
    
    /**
     * SpringJmsConsumer calls this method when it receives messages from the
     * external services about the status of an order.  We store the updated
     * status and send a message along to the client side.
     */
    public void updateClientStatus(OrderStatus status) {
        // Update the order status, and send a message to inform the client.
        if (producer != null) {            
            if (status != null) {
                producer.setTargetObject(status);
                producer.send();
            }
        }
    }
    
}
