package com.iona.fuse.demo.logisticx.service.warehouse;

/**
 * 
 * Copyright  2007-2008 IONA Technologies PLC.
 *           All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import com.iona.fuse.demo.logisticx.model.StockItem;

/** 
 * A helper class that is the worker bee for the Warehouse Service. 
 * 
 */
public class WarehouseHelper {

    private static long productId[] = {1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009};
    
    /** A JPA query to retrieve the order status based on the order id. **/    
    private static final String STOCK_QUERY = 
        "select stock from com.iona.fuse.demo.logisticx.model.StockItem stock where stock.productId = ?1";

    public WarehouseHelper() {  
        StockItem stockItem = JpaRetrieveStockItem(1002);
        if (stockItem == null) {
            for (int i = 0; i < productId.length; i++) {
                StockItem item = new StockItem();
                item.setProductId(productId[i]);
                item.setQuantity(20);
                persistStockItem(item);
            }
        }
    }
   
    /** 
     * Persists the StockItem in the database for additional downstream processing. 
     *  
     * @param object The StockItem to persist
     */
    private void persistStockItem(Object object){
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("warehouse");        
        EntityManager em = emFactory.createEntityManager();
        try {
            em.getTransaction().begin();
            if (object instanceof StockItem) {
                em.persist((StockItem) object);
            }
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
    }        
    
    /**
     * Retrieve Product Information from the database
     * based on the given <tt>itemId</tt>
     * 
     * @param itemId
     * @return StockItem
     */
    public StockItem JpaRetrieveStockItem(long itemId) {
        StockItem item = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("warehouse");
        EntityManager em = emFactory.createEntityManager();

        try {
            em.getTransaction().begin();                       
            Query stockQuery = em.createQuery(STOCK_QUERY);
            stockQuery.setParameter(1, itemId);
            item = (StockItem) stockQuery.getSingleResult();
            em.getTransaction().commit();
        } catch (org.apache.openjpa.persistence.NoResultException ex) {
          return null;
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return item;
    }


    /**
     * Retrieve Product Information from the database
     * based on the given <tt>itemId</tt>
     * Reduce Stock count by number of items ordered.
     * 
     * @param itemId
     * @return StockItem
     */
    public StockItem JpaUpdateOrderStockItem(long itemId, int amt) {
        StockItem item = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("warehouse");
        EntityManager em = emFactory.createEntityManager();

        try {
            em.getTransaction().begin();                       
            Query stockQuery = em.createQuery(STOCK_QUERY);
            stockQuery.setParameter(1, itemId);
            item = (StockItem) stockQuery.getSingleResult();
            int amount = item.getQuantity();
            amount = amount - amt;
            item.setQuantity(amount);
            em.merge(item);
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return item;
    }
    
 /**
     * Retrieve Product Information from the database
     * based on the given <tt>itemId</tt>
     * add new qty of Stock that has arrived
     * 
     * @param itemId
     * @return StockItem
     */
    public StockItem JpaUpdateNewProductStockItem(long itemId, int amt) {
        StockItem item = null;
        EntityManagerFactory emFactory = Persistence.createEntityManagerFactory("warehouse");
        EntityManager em = emFactory.createEntityManager();

        try {
            em.getTransaction().begin();                       
            Query stockQuery = em.createQuery(STOCK_QUERY);
            stockQuery.setParameter(1, itemId);
            item = (StockItem) stockQuery.getSingleResult();
            int amount = item.getQuantity();
            amount = amount + amt;
            item.setQuantity(amount);
            em.merge(item);
            em.getTransaction().commit();
        } catch (Exception e) { 
            e.printStackTrace();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
            emFactory.close();
        }
        
        return item;
    }
    
}
